#!/bin/bash
## To run this script successfully, you MUST have sourced the target machine's specific environment file
## or have exported ALL the required variables. (See subdirectories of this directory.)
## This script assumes to be run by the root user (with /sbin in the path).

required_variables="
CERT_NAME
SERVER_GROUP
"
for variable in $required_variables; do
	if [ ! -v "$variable" ]; then
		echo "ERROR: $variable is undefined, please export all required variables."
		echo "Required variables (and their value):"
		for v in $required_variables; do
			echo $v=${!v}
		done
		exit 1
	fi
done

set -e

# get the root of the git repository (requires git to be installed)
GIT_ROOT=`git rev-parse --show-toplevel`
cd $GIT_ROOT

# define useful variables
source generation/00env.sh

get_partition_path () {
	# Return the path of the desired partition.
	# Exit and display an error if it finds 0 or several results.
	#
	# This function should be called ith exactly one argument: the desired partition type.
	# Its behavior is undefined otherwise.

	desired_partitions=$(fdisk -l | grep "$1")

	desired_partition_count=$(echo -e "$desired_partitions" | wc | awk '{print $1}')
	if [ -z "$desired_partitions" ]; then
		echo -e "No partition of type $1 (shouldbe 1). Exiting."
		exit
	elif [ "$desired_partition_count" -ne 1 ]; then
		echo -e "$desired_partition_count partitions of type $1 (should be 1). Exiting."
		exit
	else
		echo -e "$desired_partitions" | awk '{print $1}'
	fi
}

get_partition_id () {
	desired_partition_path=$(get_partition_path "$1")
	echo -e "$(findmnt -fn -o UUID "$desired_partition_path")"
}

get_default_interface () {
	default_results=$(ip route | grep default | wc | awk '{print $1}')
	if [ "$default_results" -ne 1 ]; then
		echo -e "$default_results default result(s) (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep default | sed 's|.*dev \([^ ]*\).*|\1|g')"
	fi
}

get_default_router () {
	default_results=$(ip route | grep default | wc | awk '{print $1'})
	if [ "$default_results" -ne 1 ]; then
		echo -e "$defaults_results default result(s) (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep default | sed 's|.*via \([^ ]*\).*|\1|g')"
	fi
}

get_default_ip () {
	default_interface=$(get_default_interface)
	interface_results=$(ip route | grep -v default | grep "$default_interface" | wc | awk '{print $1}')
	if [ "$interface_results" -ne 1 ]; then
		echo -e "$interface_results results for \"${default_interface}\" (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep -v default | grep "$default_interface" | sed 's|.*src \([^ ]*\).*|\1|g')"
	fi
}

get_default_netmask () {
	default_interface=$(get_default_interface)
	interface_results=$(ip route | grep -v default | grep "$default_interface" | wc | awk '{print $1}')
	if [ "$interface_results" -ne 1 ]; then
		echo -e "$interface_results results for \"${default_interface}\" (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep -v default | grep "$default_interface" | sed 's|^\([^ ]*\).*|\1|g')"
	fi
}

int_to_generic_netmask () {
	# get the mask size: the number right after the "/" (ex: <ipv4>/24 -> 24)
	mask_length=$1
	mask=""
	for i in {0..3}; do
		if [ ${mask_length} -ge 8 ]; then
			mask=${mask}255
		elif [ ${mask_length} -eq 0 ]; then
			mask=${mask}0
		else
			mask=${mask}$((256-2**(8-${mask_length})))
		fi
		if [ $i -ne 3 ]; then
			mask=${mask}.
		fi
		mask_length=$((mask_length-8))
	done
	echo -e "${mask}"
}

CERT_DIR=uefi-keys/public-certificates
KEYS_DIR=uefi-keys/private-keys
PUBLIC_CERT_DIR=${PROJECT_DIR}/${CERT_DIR}/${SERVER_GROUP}
PRIVATE_KEYS_DIR=${PROJECT_DIR}/${KEYS_DIR}/${SERVER_GROUP}
dracut_output_file=dracut-output

# Install the module
cd dracut.module
./configure
make uninstall clean
make install
cd -

# Generation of dracut.module/dracut.conf
dracut_conf_regex="s|%PROJECT_DIR%|${PROJECT_DIR}|g;s|%PUBLIC_CERT_DIR%|${PUBLIC_CERT_DIR}|g;s|%PRIVATE_KEYS_DIR%|${PRIVATE_KEYS_DIR}|g;s|%CERT_NAME%|${CERT_NAME}|;s|%SIGNING_KEY_NAME%|${SIGNING_KEY_NAME}|g;s|%ROOT_PARTITION_ID%|${ROOT_PARTITION_ID}|g;s|%DEFAULT_INTERFACE%|${DEFAULT_INTERFACE}|g;s|%DEFAULT_ROUTER%|${DEFAULT_ROUTER}|g;s|%DEFAULT_IP%|${DEFAULT_IP}|g;s|%FORMATTED_NETMASK%|${FORMATTED_NETMASK}|g;s|%TARGET_HOSTNAME%|${TARGET_HOSTNAME}|g;s|%KERNEL_IMAGE%|${KERNEL_IMAGE}|g;s|%DRIVERS_DIR%|${DRIVERS_DIR}|g"
cd dracut.module
sed "${dracut_conf_regex}" dracut.conf.in > dracut.conf
cd -

# Add files in initramfs
INCLUDE_DIR=files-to-include
rm -rf "$INCLUDE_DIR"
mkdir -p "$INCLUDE_DIR"
cp -r "$KERNEL_DIR"/lib "$INCLUDE_DIR"
export DRACUT_KMODDIR_OVERRIDE=1

# Create an initramfs image
rm -f "$dracut_output_file"
dracut "$UEFI_IMAGE_NAME" "$KERNEL_VERSION" -c ./dracut.module/dracut.conf --force --include "$INCLUDE_DIR" / |& tee -a "$dracut_output_file"
echo New initramfs image: $(pwd)/"$UEFI_IMAGE_NAME"
